/**
 * FLL - Level 3
 *
 * Project: UTF8
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common define types.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _utf8_common_define_h
#define _utf8_common_define_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The program allocation defines.
 *
 * utf8_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   An allocation step used for buffers that are anticipated to have large buffers.
 *   - pipe:    A buffer size used for processing piped data.
 *   - small:   An allocation step used for buffers that are anticipated to have small buffers.
 */
#ifndef _di_utf8_allocation_d_
  #define utf8_allocation_console_d 4
  #define utf8_allocation_large_d   256
  #define utf8_allocation_pipe_d    16384
  #define utf8_allocation_small_d   16
#endif // _di_utf8_allocation_d_

/**
 * Flags passed to the main function or program.
 *
 * utf8_main_flag_*_d:
 *   - none:                   No flags set.
 *   - copyright:              Print the copyright.
 *   - file_from:              Using a specified source file.
 *   - file_to:                Using a specified destination file.
 *   - header:                 Enable printing of headers.
 *   - help:                   Print help.
 *   - pipe:                   Use the input pipe.
 *   - separate:               Enable printing of separators.
 *   - strip_invalid:          Using strip invalid character mode.
 *   - verify:                 Using verify mode.
 *   - version:                Print version.
 *   - version_copyright_help: A helper flag representing version, copyright, and help flag bits being set.
 */
#ifndef _di_utf8_main_flag_d_
  #define utf8_main_flag_none_d                   0x0
  #define utf8_main_flag_copyright_d              0x1
  #define utf8_main_flag_file_from_d              0x2
  #define utf8_main_flag_file_to_d                0x4
  #define utf8_main_flag_header_d                 0x8
  #define utf8_main_flag_help_d                   0x10
  #define utf8_main_flag_pipe_d                   0x20
  #define utf8_main_flag_separate_d               0x40
  #define utf8_main_flag_strip_invalid_d          0x80
  #define utf8_main_flag_verify_d                 0x100
  #define utf8_main_flag_version_d                0x200
  #define utf8_main_flag_version_copyright_help_d 0x211
#endif // _di_utf8_main_flag_d_

/**
 * Modes used to designate how to the input and output are to be processed.
 *
 * utf8_mode_*_d:
 *   - none: No flags set.
 *
 * utf8_mode_from_*_d:
 *   - bytesequence: The input format is bytesequence.
 *   - codepoint:    The input format is codepoint (U+XXXX or U+XXXXXX).
 *
 * utf8_mode_to_*_d:
 *   - bytesequence: The output format is bytesequence.
 *   - codepoint:    The output format is codepoint (U+XXXX or U+XXXXXX).
 *   - combining:    The output format is whether or not character is combining (may be used with "width").
 *   - width:        The output format is how wide the character is (may be used with "combining").
 */
#ifndef _di_utf8_modes_d_
  #define utf8_mode_none_e              0x0
  #define utf8_mode_from_bytesequence_e 0x1
  #define utf8_mode_from_codepoint_e    0x2
  #define utf8_mode_to_bytesequence_e   0x4
  #define utf8_mode_to_codepoint_e      0x8
  #define utf8_mode_to_combining_e      0x10
  #define utf8_mode_to_width_e          0x20
#endif // _di_utf8_modes_d_

/**
 * Flags for fine-tuned print control.
 *
 * utf8_print_flag_*_d:
 *   - none:    No flags set.
 *   - debug:   Stream is for debug printing.
 *   - error:   Stream is for error printing.
 *   - in:      Stream is a source file.
 *   - message: Stream is for message printing.
 *   - out:     Stream is a destination file.
 *   - warning: Stream is for warning printing.
 */
#ifndef _di_utf8_print_flag_d_
  #define utf8_print_flag_none_d    0x0
  #define utf8_print_flag_debug_d   0x1
  #define utf8_print_flag_error_d   0x2
  #define utf8_print_flag_file_d    0x4
  #define utf8_print_flag_in_d      0x8
  #define utf8_print_flag_out_d     0x10
  #define utf8_print_flag_message_d 0x20
  #define utf8_print_flag_warning_d 0x40
#endif // _di_utf8_print_flag_d_

/**
 * The program signal defines.
 *
 * utf8_signal_*_d:
 *   - check:          Number of iterations before performing signal check in non-threaded signal handling.
 *   - check_failsafe: When using threads, how many consecutive failures to check signal before aborting (as a recursion failsafe).
 *   - check_tiny:     The tiny check.
 *   - check_short:    The short signal check.
 */
#ifndef _di_utf8_signal_d_
  #define utf8_signal_check_d          500000
  #define utf8_signal_check_failsafe_d 20000
  #define utf8_signal_check_tiny_d     4
  #define utf8_signal_check_short_d    16
#endif // _di_utf8_signal_d_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _utf8_common_define_h
