/**
 * FLL - Level 3
 *
 * Project: UTF8
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common enumeration types.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _utf8_common_enumeration_h
#define _utf8_common_enumeration_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Codepoint modes for converting to/from binary and codepoint values.
 *
 * The special "raw" format is used only for reading from codepoint format where that format represents binary character that is not a valid Unicode character.
 * This is intended to be used to save and restore the original binary data even if that data is invalid.
 *
 * private_utf8_codepoint_mode_*:
 *   - ready:      The codepoint has yet to be processed, skip leading spaces until first 'U' is matched.
 *   - begin:      The first 'U' is matched, look for the '+'.
 *   - number:     The '+' is matched, process numbers.
 *   - end:        The last number is reached (at either white space or EOS/EOF).
 *   - bad:        This is not a valid codepoint.
 *   - bad_begin:  This is the beginning of an invalid codepoint.
 *   - bad_end:    The end of bad is detected, which happens on white space or end of buffer.
 *   - raw_begin:  This is the beginning of a potential raw data (matched '0').
 *   - raw_number: This is the confirmed beginning of raw data (matched 'X'), process numbers.
 *   - raw_end:    The end of raw data is detected, which happens on white space or end of buffer.
 */
#ifndef _di_utf8_codepoint_modes_
  enum {
    utf8_codepoint_mode_ready_e = 1,
    utf8_codepoint_mode_begin_e,
    utf8_codepoint_mode_number_e,
    utf8_codepoint_mode_end_e,
    utf8_codepoint_mode_bad_e,
    utf8_codepoint_mode_bad_begin_e,
    utf8_codepoint_mode_bad_end_e,
    utf8_codepoint_mode_raw_begin_e,
    utf8_codepoint_mode_raw_number_e,
    utf8_codepoint_mode_raw_end_e,
  }; // enum
#endif // _di__utf8_codepoint_modes_

/**
 * The main program parameters.
 */
#ifndef _di_utf8_parameter_e_
  enum {
    utf8_parameter_from_bytesequence_e = f_console_standard_parameter_last_e,
    utf8_parameter_from_codepoint_e,
    utf8_parameter_from_file_e,

    utf8_parameter_headers_e,
    utf8_parameter_separate_e,
    utf8_parameter_strip_invalid_e,

    utf8_parameter_to_bytesequence_e,
    utf8_parameter_to_codepoint_e,
    utf8_parameter_to_combining_e,
    utf8_parameter_to_file_e,
    utf8_parameter_to_width_e,

    utf8_parameter_verify_e,
  }; // enum

  #define utf8_console_parameter_t_initialize \
    { \
      macro_fll_program_console_parameter_standard_initialize, \
      \
      macro_f_console_parameter_t_initialize_3(utf8_short_from_bytesequence_s, utf8_long_from_bytesequence_s, 0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_from_codepoint_s,    utf8_long_from_codepoint_s,    0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_from_file_s,         utf8_long_from_file_s,         1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_headers_s,           utf8_long_headers_s,           0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_separate_s,          utf8_long_separate_s,          0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_strip_invalid_s,     utf8_long_strip_invalid_s,     0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_to_bytesequence_s,   utf8_long_to_bytesequence_s,   0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_to_codepoint_s,      utf8_long_to_codepoint_s,      0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_to_combining_s,      utf8_long_to_combining_s,      0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_to_file_s,           utf8_long_to_file_s,           1, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_to_width_s,          utf8_long_to_width_s,          0, f_console_flag_normal_e), \
      macro_f_console_parameter_t_initialize_3(utf8_short_verify_s,            utf8_long_verify_s,            0, f_console_flag_normal_e), \
    }

  #define utf8_parameter_total_d (f_console_parameter_state_type_total_d + 12)
#endif // _di_utf8_parameter_e_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _utf8_common_enumeration_h
